#!/usr/bin/php
<?php

use Logger\Log;
use Microsoft\Office365;
use Wildix\Core\Container;
use Wildix\Core\Notification;
use Wildix\Error;
use Wildix\Exception;
use Wildix\Faxes\FaxSmsProfiles;

require_once '/var/www/init.php';

/**
 * @var Log $logger
 */
$logger = Container::get(Log::class);

/**
 * @var FaxSmsProfiles $faxSmsProfiles
 */
$faxSmsProfiles = Container::get(FaxSmsProfiles::class);

$accessTokenFiles = glob(sprintf(FaxSmsProfiles::OAUTH_CONFIG_ACCESS_FILE_PATTERN, '*'));

foreach ($accessTokenFiles as $file) {
    if (isNeedRefresh($file) && preg_match('/_(\d+)/', $file, $matches)) {
        $profileId = $matches[1];
        $logger->info(__LINE__, sprintf('Access token for profile id %d should be updated', $profileId));

        $profile = $faxSmsProfiles->getProfile($profileId);

        try {
            $provider = getOauthProvider($profile['authProvider']);

            $token = $provider->refreshToken(
                [
                    'refresh_token' => getRefreshToken($profileId)
                ]
            );

            $faxSmsProfiles->writeOauthConfigFiles(
                [
                    'profileId'    => $profileId,
                    'accessToken'  => $token['access_token'],
                    'refreshToken' => $token['refresh_token'],
                ]
            );

            $logger->info(__LINE__, sprintf('Access token for profile id %d updated successfully', $profileId));
        } catch (Exception $e) {
            $error = sprintf('Access token for profile id %d update failed. Reason: %s', $profileId, $e->getMessage());
            $logger->error(__LINE__, $error);
            $faxSmsProfiles->removeOauthConfigFiles((int)$profileId);
            sendAdminEmail($error);
        }
    }
}

function isNeedRefresh(string $file): bool {
    if (!file_exists($file) || !is_writable($file)) {
        return false;
    }

    $lifetime = time() - filemtime($file);

    return (FaxSmsProfiles::OAUTH_ACCESS_EXPIRATION_SECONDS - $lifetime) <= FaxSmsProfiles::OAUTH_ACCESS_TOKEN_REFRESH_LIMIT;
}

function getRefreshToken(int $profileId): string {
    $file = sprintf(FaxSmsProfiles::OAUTH_CONFIG_REFRESH_FILE_PATTERN, $profileId);
    if (!file_exists($file) || !is_writable($file)) {
        throw new Exception(Error::getMessage(Error::IsNotWriteable) . ' ' . $file);
    }

    return file_get_contents($file);
}

function getOauthProvider(string $provider): object {
    if ($provider === User::AUTH_TYPE_GOOGLE) {
        return GoogleAPI::getInstance();
    } else if ($provider === User::AUTH_TYPE_MICROSOFT) {
        return Container::get(Office365::class);
    } else {
        throw new \Exception('Unknown auth provider');
    }
}

function sendAdminEmail(string $body): void {
    $adminMail = Container::get(WMS_Ldap::class)->getEntry(WMS_Ldap::PERSONS, '(cn=admin)', ['mail'])['mail'] ?? '';

    if (filter_var($adminMail, FILTER_VALIDATE_EMAIL)) {
        $params = array(
            'type' => '',
            'subject' => '',
            'to' => $adminMail,
            'body' => $body,
            'template' => '',
            'attach' => ''
        );

        Notification::sendEmail($params);
    }
}
