#!/bin/bash

CONF_FILE='/rw2/etc/pbx/trunk_notification.conf'
CALLWEAVER_PID='/var/run/callweaver/callweaver.pid'
DEBUG='0'


__LOG() {

  local name message xargs

  name="${0##*/}"
  message="${2}"
  xargs="${3}"

  case "${1}" in
    crit)   severity='crit'    ;;
    err)    severity='err'     ;;
    warn)   severity='warning' ;;
    notice) severity='notice'  ;;
    debug)  severity='debug'   ;;
    *)      severity='info'    ;;
  esac

  if [ "${message}" ]; then
    echo ${xargs} "${message}" | tee >(systemd-cat -t "${name}" -p "${severity}")
  else
    systemd-cat -t "${name}" -p "${severity}"
  fi

}


__SEND_MAIL(){

  local mail_config mail_from mail_admin pbx_name mailbody subject_status sip_status arr

    ## include script for ldap
  if [ -s '/usr/sbin/ldapcmd' ]; then
    source /usr/sbin/ldapcmd
  else
    __LOG "err" "ERROR($FUNCNAME): /usr/sbin/ldapcmd was not found." && return 1
  fi

  declare -n arr=${1}
  mail_config='/etc/companies.d/0/msmtprc'
  mail_from="$(awk '/from/{print $2}' ${mail_config})"
  mail_admin="$(__LDAP_GET "(&(objectClass=wPerson)(cn=admin)(userKind=admin)) mail||mail" "dc=wildix" | awk '{print $2}')"
  pbx_name="$(cat /etc/hostname)"

    # check mail password and admin mail
  [ -z "${mail_admin}" ] && \
    __LOG "err" "ERROR($FUNCNAME): not set admin email" && return 1

    # check from
  [ -z "${mail_from}" ] && mail_from="no-reply@wildix.com"

    # select a subject
  [ "${arr['mail_status']}" == 'up' ] && subject_status="Notification" || subject_status="Warning"

    # send email
  mailbody="sip: ${arr['domain']} (${pbx_name})\naccount: ${arr['account']}\ntime: $(date)\nstatus: ${arr['status']}"

  /usr/sbin/sendemail.php \
    --type="default" \
    --body="$(echo -e ${mailbody})" \
    --subject="${subject_status} ${pbx_name}: ${arr['domain']} (${arr['account']}) is ${arr['mail_status']}!" \
    --to=${mail_admin}

  [ "${?}" -eq "0" ] && __LOG "info" "Notification has been sent to ${mail_admin}." || \
    __LOG "err" "ERROR($FUNCNAME): Notification hasn't been delivered due to an error to ${mail_admin}."

}


__FLAG(){

  local status_file arr attempts_up attempts_down attempts_request attempts_limit attempts

  declare -n arr=${1}
  attempts_up='2'
  attempts_down='2'
  attempts_request='10'
  attempts_limit='10'
  status_file="/tmp/sip_trunk_$(md5sum <<< "${arr['name']}_${arr['domain']}_${arr['account']}" | awk '{print $1}').lock"

  [ -s "${status_file}" ] && source ${status_file} 2>/dev/null # load array for counters

    ## load array for sip trunk
  if [ "${#attempts[@]}" -eq '3' ]; then
      # checking correct data
    for i in ${!attempts[@]}; do
      [[ ! "${attempts[${i}]}" =~ ^[0-9]{1,2}$ ]] && attempts[${i}]=0
    done
  else
      # declare a new array for counters
    [ -f "${status_file}" ] && rm -f ${status_file}
    declare -A attempts=(
        ['up']=0
        ['down']=0
        ['request']=0
      )
  fi

    ## handle trunk state
  case "${arr['status']}" in
    Registered)
      if [ "${attempts['down']}" -gt '0' ] || [ "${attempts['request']}" -gt '0' ]; then

        ((attempts['up']++))
        if [ "${attempts['up']}" -eq "${attempts_up}" ]; then

          rm -f ${status_file}
          if [ "${attempts['down']}" -ge "${attempts_down}" ] || [ "${attempts['request']}" -ge "${attempts_request}" ]; then
            echo "up"
          fi

        else
          declare -p attempts > ${status_file}
        fi

      fi
    ;;
    Request_Sent|Auth._Sent)
      if [ "${attempts['request']}" -le "${attempts_limit}" ]; then

        ((attempts['request']++))
        declare -p attempts > ${status_file}
        [ "${attempts['request']}" -eq "${attempts_request}" ] && echo "down"

      fi
    ;;
    *)
      if [ "${attempts['down']}" -le "${attempts_limit}" ]; then

        ((attempts['down']++))
        declare -p attempts > ${status_file}
        [ "${attempts['down']}" -eq "${attempts_down}" ] && echo "down"

      fi
    ;;
  esac

}


  ## check conf file
if [ ! -f "${CONF_FILE}" ]; then
  cat << 'EOF' > ${CONF_FILE}
[general]
start=0
EOF
fi

  ## exit if start is 0
[ "$(awk -F '=' '/start=/{print $2}' ${CONF_FILE})" == "0" ] && exit
  ## exit if file pid is less than 120 seconds after creation
[ ! -f "${CALLWEAVER_PID}" ] || [ "$(( $(date +%s) - $(stat -c %Y ${CALLWEAVER_PID}) ))" -le '120' ] && exit


CALLWEAVER_ROW_DATA="$(/usr/sbin/callweaver -rx 'sip show registry' 2>/dev/null)"
CALLWEAVER_REQUEST_STATUS="${?}"

if [ "${CALLWEAVER_REQUEST_STATUS}" -eq '0' ] && [ "$(grep -oE 'SIP registrations' <<< ${CALLWEAVER_ROW_DATA})" ]; then

  sed -e "s/Request Sent/Request_Sent/g" \
      -e "s/Auth. Sent/Auth._Sent/g" \
      -e "s/No Authentication/No_Authentication/g" \
      -e "s/ws2sip.wildix.com/kite-connection/g" \
      -e "/Reg.Time\|SIP registrations/d" <<< "${CALLWEAVER_ROW_DATA}" | \
  awk '{print $1" "$3" "$4" "$5" "$6" "$7"_"$8"_"$9"_"$10"_"$11}' | \
  while read DOMAIN ACCOUNT NAME REFRESH STATUS LAST_REGISTRATION; do

    declare -A SIP=(
        ['name']="${NAME}"
        ['domain']="${DOMAIN}"
        ['account']="${ACCOUNT}"
        ['refresh']="${REFRESH}"
        ['status']="${STATUS}"
        ['mail_status']="$([ "${STATUS}" == 'Registered' ] && echo "up" || echo "down")"
        ['last_registration']="${LAST_REGISTRATION}"
      )

    case "$(__FLAG SIP)" in
        up) __SEND_MAIL SIP ;;
      down) __SEND_MAIL SIP ;;
    esac

    [ "${DEBUG}" -eq '1' ] && \
      __LOG "notice" "debug: name:[${SIP['name']}], domain:[${SIP['domain']}], account:[${SIP['account']}], refresh:[${SIP['refresh']}], status:[${SIP['status']}], last registrations:[${SIP['last_registration']}]"

  done

else
  __LOG "err" "ERROR: callweaver returned invalid data or an error (${CALLWEAVER_REQUEST_STATUS}). Exit."
fi
