import translate from '@/modules/translate/translate';
import cloneDeep from 'lodash/cloneDeep';

import Errors from '@/pages/Errors.vue';
import Security from '@/pages/security/Security.vue';
import DHCP from '@/pages/dhcp/Dhcp.vue';
import SystemLdapServer from '@/pages/ldapserver/SystemLdapServer.vue';
import Faxes from '@/pages/faxes/Faxes.vue';
import Ntp from '@/pages/ntp/Ntp.vue';
import Smtp from '@/pages/smtp/Smtp.vue';
import Upgrade from '@/pages/upgrade/Upgrade.vue';
import License from '@/pages/license/License.vue';
import Snmp from '@/pages/snmp/Snmp.vue';
import Integrations from '@/pages/integrations/Integrations.vue';
import HttpProxy from '@/pages/httpproxy/HttpProxy.vue';
import RemoteSyslog from '@/pages/remoteSysLog/index.vue';
import Trace from '@/pages/trace/index.vue';
import NetworkSettings from '@/pages/network/NetworkSettings.vue';
import HotelIntegration from '@/pages/hotelIntegration/HotelIntegration.vue';
import BackupSystem from '@/pages/backupSystem/BackupSystem.vue';
import Storages from '@/pages/storages/Storages.vue';
import RemoteSupport from '@/pages/remoteSupport/RemoteSupport.vue';
import Debug from '@/pages/debug/Debug.vue';
import Trunks from '@/pages/trunks/index.vue';
import Users from '@/pages/users/index.vue';
import Devices from '@/pages/devices/index.vue';
import Dialplans from '@/pages/dialplans/index.vue';
import WmsNetwork from '@/pages/wmsNetwork/WmsNetwork.vue';
import Language from '@/pages/language/Language.vue';
import CallChatHistory from '@/pages/callChatHistory/CallChatHistory.vue';
import SipRtp from '@/pages/sipRtp/SipRtp.vue';
import Features from '@/pages/features/Features.vue';

import {
    SETTING_ACTIVATION,
    SETTING_SYSTEM,
    SETTING_PBX,
    SETTING_BACKUP,
    SETTING_UPGRADE,
    SETTINGS_UTILITIES_GENERATE_TRACE
} from '@/configuration/acl';

import {verifyAccess} from './middleware';

/**
 * To add access validation:
 * {
 *     path: ....
 *     name: .....,
 *     meta: {
 *        ....,
 *        middleware: [verifyAccess],
 *        resource: SETTING_SYSTEM
 *    }
 * }
 * To add virtual pbx validation
 * {
 *     path: ....
 *     name: .....,
 *     meta: {
 *        ....,
 *        middleware: [verifyAccess],
 *        virtualOnly: true
 *    }
 * }
 * To add master pbx validation
 * {
 *     path: ....
 *     name: .....,
 *     meta: {
 *        ....,
 *        middleware: [verifyAccess],
 *        masterOnly: true
 *    }
 * }
 * !!!!!! Note !!!!!!!!
 * in case router has few middlewares, order is important. verifyAccess should be last
 * To disable route on default password or not activated licence add
 * meta: {
 *    ....,
 *     disableOnNotActive: true
 * }
 */

const prepareMenu = (items, isCloud, isSlave) => {
    return items.filter(item => {
        if (item.items?.length) {
            item.items = prepareMenu(item.items, isCloud, isSlave);
        }

        return isAvailable(item, isCloud, isSlave) && !item.hidden;
    });
};

const prepareRoutes = (items, isCloud, isSlave) => {
    let routes = [];
    items.forEach(item => {
        if(item.hasOwnProperty('component') && isAvailable(item, isCloud, isSlave)) {
            const {link, name, component, meta, redirect, props} = item;
            routes.push({path: link, name, component, meta, redirect, props});
        }
        if(item.items.length) {
            routes = routes.concat(prepareRoutes(item.items, isCloud, isSlave, isSlave));
        }
    });

    return routes;
};

const isAvailable = (item, isCloud, isSlave) => (isCloud ? item.inCloud === isCloud : true) && (!item.isMasterOnly || !isSlave);

const routes = [
    // {path: '/', redirect: '/users', hidden: true},
    {
        link: '/',
        name: 'home',
        inCloud: true,
        hidden: true,
        component: Users,
        redirect: '/users',
        items: []
    },
    {
        link: '/users',
        name: 'users',
        icon: 'fa fa-users',
        inCloud: true,
        hidden: false,
        component: Users,
        title: translate('menu.users'),
        items: []
    },
    {
        link: '/trunks',
        name: 'trunks',
        icon: 'fa fa-exchange',
        inCloud: true,
        hidden: false,
        component: Trunks,
        meta: {
            disableOnNotActive: true
        },
        title: translate('menu.trunks'),
        items: []
    },
    {
        link: '/devices',
        name: 'devices',
        icon: 'fa fa-fax',
        inCloud: true,
        hidden: false,
        component: Devices,
        meta: {
            middleware: [verifyAccess],
            resource: 'devices',
            disableOnNotActive: true
        },
        title: translate('menu.devices'),
        items: []
    },
    {
        link: '/dialplans',
        name: 'dialplans',
        icon: 'fa fa-sitemap',
        inCloud: true,
        component: Dialplans,
        meta: {
            disableOnNotActive: true
        },
        title: translate('menu.dialplan'),
        items: []
    },
    {
        title: translate('menu.pbx_settings'),
        name: 'pbx',
        icon: 'fa fa-phone-web',
        inCloud: true,
        hidden: false,
        items: [
            {
                link: '/wms-network',
                name: 'wms-network',
                icon: '',
                inCloud: true,
                hidden: false,
                component: WmsNetwork,
                title: translate('menu.pbxnet_settings'),
                items: []
            },
            {
                link: '/language',
                name: 'language',
                icon: '',
                inCloud: true,
                hidden: false,
                component: Language,
                title: translate('menu.language_settings'),
                meta: {
                    middleware: [verifyAccess],
                    resource: SETTING_PBX,
                    disableOnNotActive: true
                },
                items: []
            },
            {
                link: '/call-chat-history',
                name: 'call-chat-history',
                icon: '',
                inCloud: true,
                hidden: false,
                component: CallChatHistory,
                meta: {
                    disableOnNotActive: true
                },
                title: translate('menu.cdrconfig'),
                items: []
            },
            {
                link: '/sip-rtp',
                name: 'sip-rtp',
                icon: '',
                inCloud: true,
                hidden: false,
                component: SipRtp,
                meta: {
                    disableOnNotActive: true
                },
                title: translate('menu.sip_rtp_codecs'),
                items: []
            },
            {
                link: '/hotel-integration',
                name: 'hotel-integration',
                icon: '',
                inCloud: true,
                hidden: false,
                component: HotelIntegration,
                meta: {
                    middleware: [verifyAccess],
                    resource: SETTING_PBX,
                    disableOnNotActive: true
                },
                title: translate('menu.hotel_integration'),
                items: []
            },
            {
                link: '/integrations',
                name: 'integrations',
                icon: '',
                inCloud: true,
                hidden: false,
                component: Integrations,
                title: translate('menu.crm_integrations'),
                items: []
            },
            {
                link: '/features',
                name: 'features',
                icon: '',
                inCloud: true,
                hidden: false,
                component: Features,
                title: translate('menu.pbx_features'),
                meta: {
                    middleware: [verifyAccess],
                    resource: SETTING_PBX,
                    disableOnNotActive: true
                },
                items: []
            },
            {
                link: '/security',
                name: 'security',
                icon: '',
                inCloud: true,
                hidden: false,
                component: Security,
                meta: {
                    middleware: [verifyAccess],
                    resource: SETTING_SYSTEM
                },
                title: translate('menu.security_settings'),
                items: []
            }
        ]
    },
    {
        title: translate('menu.system_settings'),
        name: 'system',
        icon: 'fa fa-cogs',
        inCloud: true,
        hidden: false,
        items: [
            {
                name: 'network-settings',
                component: NetworkSettings,
                icon: '',
                inCloud: false,
                hidden: false,
                link: '/network',
                title: translate('menu.netsettings'),
                meta: {
                    middleware: [verifyAccess],
                    resource: SETTING_SYSTEM,
                    virtualOnly: true
                },
                items: []
            },
            {
                link: '/httpproxy',
                name: 'httpproxy',
                icon: '',
                inCloud: false,
                hidden: false,
                component: HttpProxy,
                meta: {
                    middleware: [verifyAccess],
                    resource: SETTING_SYSTEM
                },
                title: translate('menu.proxy_settings'),
                items: []
            },
            {
                link: '/ldapserver',
                name: 'ldapserver',
                icon: '',
                inCloud: true,
                isMasterOnly: true,
                hidden: false,
                component: SystemLdapServer,
                meta: {
                    middleware: [verifyAccess],
                    resource: SETTING_SYSTEM,
                    masterOnly: true,
                    disableOnNotActive: true
                },
                title: translate('menu.ldap_net_settings'),
                items: []
            },
            {
                link: '/dhcp',
                name: 'dhcp',
                icon: '',
                inCloud: false,
                hidden: false,
                component: DHCP,
                meta: {
                    middleware: [verifyAccess],
                    resource: SETTING_SYSTEM,
                    disableOnNotActive: true
                },
                title: translate('menu.dhcp_server'),
                items: []
            },
            {
                link: '/smtp',
                name: 'smtp',
                icon: '',
                inCloud: true,
                hidden: false,
                component: Smtp,
                meta: {
                    middleware: [verifyAccess],
                    resource: SETTING_SYSTEM,
                    disableOnNotActive: true
                },
                title: translate('menu.smtp_settings'),
                items: []
            },
            {
                link: '/faxes',
                name: 'faxes',
                icon: '',
                inCloud: true,
                hidden: false,
                component: Faxes,
                meta: {
                    middleware: [verifyAccess],
                    resource: SETTING_SYSTEM,
                    disableOnNotActive: true
                },
                title: translate('menu.fax_settings'),
                items: []
            },
            {
                link: '/snmp',
                name: 'snmp',
                icon: '',
                inCloud: false,
                hidden: false,
                component: Snmp,
                meta: {
                    middleware: [verifyAccess],
                    resource: SETTING_SYSTEM,
                    disableOnNotActive: true
                },
                title: translate('menu.snmp_settings'),
                items: []
            },
            {
                link: '/storages',
                name: 'storages',
                icon: '',
                inCloud: false,
                hidden: false,
                component: Storages,
                title: translate('menu.storages'),
                meta: {
                    middleware: [verifyAccess],
                    resource: SETTING_SYSTEM,
                    virtualOnly: true
                },
                items: []
            },
            {
                link: '/ntp',
                name: 'ntp',
                icon: '',
                inCloud: true,
                hidden: false,
                component: Ntp,
                meta: {
                    middleware: [verifyAccess],
                    resource: SETTING_SYSTEM
                },
                title: translate('menu.ntp_server'),
                items: []
            },
            {
                link: '/remotesyslog',
                name: 'remotesyslog',
                icon: '',
                inCloud: true,
                hidden: false,
                component: RemoteSyslog,
                meta: {
                    middleware: [verifyAccess],
                    resource: SETTING_SYSTEM,
                    disableOnNotActive: true
                },
                title: translate('menu.remote_syslog'),
                items: []
            }
        ]
    },
    {
        title: translate('menu.utilites'),
        name: 'toolsAndUtilities',
        icon: 'fa fa-wrench',
        inCloud: true,
        hidden: false,
        items: [
            {
                link: '/remote-support',
                name: 'remote-support',
                icon: '',
                inCloud: false,
                hidden: false,
                component: RemoteSupport,
                title: translate('menu.vtun'),
                meta: {
                    virtualOnly: true
                },
                items: []
            },
            {
                link: '/backups',
                name: 'backups',
                icon: '',
                inCloud: true,
                hidden: false,
                component: BackupSystem,
                meta: {
                    middleware: [verifyAccess],
                    resource: SETTING_BACKUP
                },
                title: translate('menu.backupsystem'),
                items: []
            },
            {
                link: '/upgrade',
                name: 'upgrade',
                icon: '',
                inCloud: true,
                hidden: false,
                component: Upgrade,
                meta: {
                    middleware: [verifyAccess],
                    resource: SETTING_UPGRADE
                },
                title: translate('menu.upgrade'),
                items: []
            },
            {
                link: '/license',
                name: 'license',
                icon: '',
                inCloud: true,
                hidden: false,
                component: License,
                meta: {
                    middleware: [verifyAccess],
                    resource: SETTING_ACTIVATION
                },
                title: translate('menu.activation'),
                items: []
            },
            {
                link: '/trace',
                name: 'trace',
                icon: '',
                inCloud: true,
                hidden: false,
                component: Trace,
                meta: {
                    middleware: [verifyAccess],
                    resource: SETTINGS_UTILITIES_GENERATE_TRACE,
                    disableOnNotActive: true
                },
                title: translate('menu.generate_trace'),
                items: []
            }
        ]
    },
    {
        link: '/debug',
        name: 'Debug',
        inCloud: true,
        hidden: true,
        component: Debug,
        items: []
    },
    {
        link: '/errors',
        name: 'errors',
        inCloud: true,
        hidden: true,
        component: Errors,
        items: []
    },
    {
        link: '*',
        name: 'permissionsDenied',
        inCloud: true,
        hidden: true,
        component: Errors,
        items: []
    }
];

export const getMenu = (isCloud) => prepareMenu(cloneDeep(routes), isCloud);

export const getRoutes = (isCloud = false, isSlave = false) => prepareRoutes(cloneDeep(routes), isCloud, isSlave);
